"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIChatView = void 0;
const tslib_1 = require("tslib");
const mobx_react_lite_1 = require("mobx-react-lite");
const React = tslib_1.__importStar(require("react"));
const react_chat_elements_1 = require("react-chat-elements");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const command_service_1 = require("@opensumi/ide-editor/lib/browser/monaco-contrib/command/command.service");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
const htmlContent_1 = require("@opensumi/monaco-editor-core/esm/vs/base/common/htmlContent");
require("react-chat-elements/dist/main.css");
const common_1 = require("../../common");
const ChatEditor_1 = require("../components/ChatEditor");
const ChatInput_1 = require("../components/ChatInput");
const ChatMarkdown_1 = require("../components/ChatMarkdown");
const ChatReply_1 = require("../components/ChatReply");
const ChatThinking_1 = require("../components/ChatThinking");
const SlashCustomRender_1 = require("../components/SlashCustomRender");
const StreamReplyRender_1 = require("../components/StreamReplyRender");
const utils_1 = require("../components/utils");
const msg_stream_manager_1 = require("../model/msg-stream-manager");
const chat_module_less_1 = tslib_1.__importDefault(require("./chat.module.less"));
const SCROLL_CLASSNAME = 'chat_scroll';
const InitMsgComponent = () => {
    var _a, _b;
    const aiChatService = (0, ide_core_browser_1.useInjectable)(common_1.IAIChatService);
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const chatRenderRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatRenderRegistryToken);
    const [sampleQuestions, setSampleQuestions] = React.useState([]);
    const welcomeSampleQuestions = React.useMemo(() => {
        if (!chatFeatureRegistry.chatWelcomeMessageModel) {
            return [];
        }
        const { sampleQuestions = [] } = chatFeatureRegistry.chatWelcomeMessageModel;
        return sampleQuestions.map(utils_1.extractIcon);
    }, [(_a = chatFeatureRegistry.chatWelcomeMessageModel) === null || _a === void 0 ? void 0 : _a.sampleQuestions]);
    const welcomeMessage = React.useMemo(() => {
        if (!chatFeatureRegistry.chatWelcomeMessageModel) {
            return '';
        }
        const { content } = chatFeatureRegistry.chatWelcomeMessageModel;
        return content;
    }, [(_b = chatFeatureRegistry.chatWelcomeMessageModel) === null || _b === void 0 ? void 0 : _b.content]);
    React.useEffect(() => {
        const disposer = chatAgentService.onDidChangeAgents(async () => {
            const sampleQuestions = await chatAgentService.getAllSampleQuestions();
            const lists = sampleQuestions.map(utils_1.extractIcon);
            setSampleQuestions(lists);
        });
        return () => disposer.dispose();
    }, []);
    if (!welcomeMessage) {
        return (React.createElement(ChatThinking_1.ChatThinking, { status: 1 /* EMsgStreamStatus.THINKING */, showStop: false, thinkingText: (0, ide_core_common_1.localize)('aiNative.chat.welcome.loading.text') }));
    }
    const welcomeRender = React.useMemo(() => {
        if (chatRenderRegistry.chatWelcomeRender) {
            return chatRenderRegistry.chatWelcomeRender({ message: welcomeMessage, sampleQuestions: welcomeSampleQuestions });
        }
        return (React.createElement("div", { className: chat_module_less_1.default.chat_head },
            React.createElement("div", { className: chat_module_less_1.default.chat_container_des }, (0, htmlContent_1.isMarkdownString)(welcomeMessage) ? React.createElement(ChatMarkdown_1.ChatMarkdown, { markdown: welcomeMessage }) : welcomeMessage),
            React.createElement("div", { className: chat_module_less_1.default.chat_container_content }, welcomeSampleQuestions.concat(sampleQuestions).map((data, index) => {
                const node = (React.createElement("a", { href: 'javascript:void(0)', className: chat_module_less_1.default.link_item, onClick: () => {
                        aiChatService.launchChatMessage(chatAgentService.parseMessage(data.message));
                    } },
                    data.icon ? React.createElement(components_1.Icon, { className: data.icon, style: { color: 'inherit', marginRight: '4px' } }) : '',
                    React.createElement("span", null, data.title)));
                return data.tooltip ? (React.createElement(components_1.Tooltip, { title: data.tooltip, key: index }, node)) : (React.createElement(React.Fragment, { key: index }, node));
            }))));
    }, [chatRenderRegistry.chatWelcomeRender, welcomeMessage, welcomeSampleQuestions]);
    return welcomeRender;
};
exports.AIChatView = (0, mobx_react_lite_1.observer)(() => {
    const aiChatService = (0, ide_core_browser_1.useInjectable)(common_1.IAIChatService);
    const aiReporter = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.IAIReporter);
    const msgStreamManager = (0, ide_core_browser_1.useInjectable)(msg_stream_manager_1.MsgStreamManager);
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const chatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatFeatureRegistryToken);
    const chatRenderRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatRenderRegistryToken);
    const monacoCommandRegistry = (0, ide_core_browser_1.useInjectable)(command_service_1.MonacoCommandRegistry);
    const layoutService = (0, ide_core_browser_1.useInjectable)(ide_main_layout_1.IMainLayoutService);
    const containerRef = React.useRef(null);
    const chatInputRef = React.useRef(null);
    const [messageListData, dispatchMessage] = React.useReducer((state, action) => {
        switch (action.type) {
            case 'add':
                return [...state, ...(action.payload || [])];
            case 'clear':
                return [];
            case 'init':
                return Array.isArray(action.payload) ? action.payload : [];
            default:
                return state;
        }
    }, []);
    const [loading, setLoading] = React.useState(false);
    const [loading2, setLoading2] = React.useState(false);
    const [agentId, setAgentId] = React.useState('');
    const [defaultAgentId, setDefaultAgentId] = React.useState('');
    const [command, setCommand] = React.useState('');
    const [theme, setTheme] = React.useState(null);
    const aiAssistantName = React.useMemo(() => (0, ide_core_common_1.localize)('aiNative.chat.ai.assistant.name'), []);
    const shortcutCommands = React.useMemo(() => chatFeatureRegistry.getAllShortcutSlashCommand(), [chatFeatureRegistry]);
    React.useEffect(() => {
        msgStreamManager.onMsgStatus((event) => {
            if (event === 2 /* EMsgStreamStatus.DONE */ || event === 3 /* EMsgStreamStatus.ERROR */) {
                setLoading2(false);
            }
            else if (event === 1 /* EMsgStreamStatus.THINKING */) {
                setLoading2(true);
            }
        });
        return () => {
            msgStreamManager.dispose();
        };
    }, []);
    const firstMsg = React.useMemo(() => (0, utils_1.createMessageByAI)({
        id: (0, ide_core_common_1.uuid)(6),
        relationId: '',
        text: React.createElement(InitMsgComponent, null),
    }), []);
    const scrollToBottom = React.useCallback(() => {
        if (containerRef && containerRef.current) {
            containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
            // 出现滚动条时出现分割线
            if (containerRef.current.scrollHeight > containerRef.current.clientHeight) {
                containerRef.current.classList.add(SCROLL_CLASSNAME);
            }
        }
    }, [containerRef]);
    React.useEffect(() => {
        dispatchMessage({ type: 'init', payload: [firstMsg] });
    }, []);
    React.useEffect(() => {
        scrollToBottom();
    }, [loading, loading2]);
    React.useEffect(() => {
        const dispose = msgStreamManager.onMsgStatus(() => {
            requestAnimationFrame(() => {
                scrollToBottom();
            });
        });
        return () => dispose.dispose();
    }, [msgStreamManager.onMsgStatus]);
    React.useEffect(() => {
        const dispose = aiChatService.onChatMessageLaunch(async (message) => {
            var _a;
            if (message.immediate !== false) {
                if (loading) {
                    return;
                }
                await handleSend(message);
            }
            else {
                if (message.agentId) {
                    setAgentId(message.agentId);
                }
                if (message.command) {
                    setCommand(message.command);
                }
                (_a = chatInputRef === null || chatInputRef === void 0 ? void 0 : chatInputRef.current) === null || _a === void 0 ? void 0 : _a.setInputValue(message.message);
            }
        });
        return () => dispose.dispose();
    }, [messageListData, loading]);
    React.useEffect(() => {
        const disposer = chatAgentService.onDidSendMessage((chunk) => {
            const relationId = aiReporter.start(ide_core_common_1.AISerivceType.Agent, {
                msgType: ide_core_common_1.AISerivceType.Agent,
                message: '',
            });
            const notifyMessage = (0, utils_1.createMessageByAI)({
                id: (0, ide_core_common_1.uuid)(6),
                relationId,
                text: React.createElement(ChatReply_1.ChatNotify, { relationId: relationId, chunk: chunk }),
            }, chat_module_less_1.default.chat_notify);
            dispatchMessage({ type: 'add', payload: [notifyMessage] });
            requestAnimationFrame(() => {
                scrollToBottom();
            });
        });
        return () => disposer.dispose();
    }, []);
    React.useEffect(() => {
        const disposer = chatAgentService.onDidChangeAgents(async () => {
            const newDefaultAgentId = chatAgentService.getDefaultAgentId();
            setDefaultAgentId(newDefaultAgentId !== null && newDefaultAgentId !== void 0 ? newDefaultAgentId : '');
        });
        return () => disposer.dispose();
    }, []);
    const handleSlashCustomRender = React.useCallback(async (value) => {
        const { message, relationId, render, startTime } = value;
        const aiMessage = (0, utils_1.createMessageByAI)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            className: chat_module_less_1.default.chat_with_more_actions,
            text: (React.createElement(SlashCustomRender_1.SlashCustomRender, { message: message, startTime: startTime, relationId: relationId, renderContent: render })),
        });
        dispatchMessage({ type: 'add', payload: [aiMessage] });
        if (containerRef && containerRef.current) {
            containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
        }
    }, [containerRef]);
    const handleAgentReply = React.useCallback(async (value) => {
        const { message, agentId, command } = value;
        const chatUserRoleRender = chatRenderRegistry.chatUserRoleRender;
        const request = aiChatService.createRequest(message, agentId, command);
        if (!request) {
            return;
        }
        const startTime = Date.now();
        const relationId = aiReporter.start(ide_core_common_1.AISerivceType.Agent, {
            msgType: ide_core_common_1.AISerivceType.Agent,
            message: value.message,
        });
        const userMessage = (0, utils_1.createMessageByUser)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            text: chatUserRoleRender ? (chatUserRoleRender({ content: message, agentId, command })) : (React.createElement(ChatEditor_1.CodeBlockWrapperInput, { relationId: relationId, text: message, agentId: agentId, command: command })),
        }, chat_module_less_1.default.chat_message_code);
        const aiMessage = (0, utils_1.createMessageByAI)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            className: chat_module_less_1.default.chat_with_more_actions,
            text: (React.createElement(ChatReply_1.ChatReply, { relationId: relationId, request: request, startTime: startTime, onRegenerate: () => {
                    msgStreamManager.sendThinkingStatue();
                    aiChatService.sendRequest(request, true);
                } })),
        });
        msgStreamManager.setCurrentSessionId(relationId);
        msgStreamManager.sendThinkingStatue();
        aiChatService.setLatestSessionId(relationId);
        aiChatService.sendRequest(request);
        dispatchMessage({ type: 'add', payload: [userMessage, aiMessage] });
        if (containerRef && containerRef.current) {
            containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
        }
    }, [chatRenderRegistry, chatRenderRegistry.chatUserRoleRender]);
    const handleSend = React.useCallback(async (value) => {
        const { message, prompt, reportType, agentId, command } = value;
        const chatUserRoleRender = chatRenderRegistry.chatUserRoleRender;
        if (agentId) {
            return handleAgentReply({ message, agentId, command });
        }
        const userInput = {
            type: ide_core_common_1.AISerivceType.Chat,
            message: prompt || message,
        };
        const { nameWithSlash } = chatFeatureRegistry.parseSlashCommand(message);
        let commandHandler;
        if (nameWithSlash) {
            commandHandler = chatFeatureRegistry.getSlashCommandHandlerBySlashName(nameWithSlash);
        }
        if (commandHandler && commandHandler.providerPrompt) {
            const editor = monacoCommandRegistry.getActiveCodeEditor();
            const slashCommandPrompt = await commandHandler.providerPrompt(message, editor);
            userInput.message = slashCommandPrompt;
        }
        const startTime = +new Date();
        const relationId = aiReporter.start(reportType || userInput.type, {
            msgType: reportType || userInput.type,
            message: userInput.message,
        });
        const sendMessage = (0, utils_1.createMessageByUser)({
            id: (0, ide_core_common_1.uuid)(6),
            relationId,
            text: chatUserRoleRender ? (chatUserRoleRender({ content: message, agentId: agentId !== null && agentId !== void 0 ? agentId : '', command: command !== null && command !== void 0 ? command : '' })) : (React.createElement(ChatEditor_1.CodeBlockWrapperInput, { relationId: relationId, text: message, agentId: agentId, command: command })),
        }, chat_module_less_1.default.chat_message_code);
        dispatchMessage({ type: 'add', payload: [sendMessage] });
        if (commandHandler && commandHandler.providerRender) {
            return handleSlashCustomRender({
                message,
                render: commandHandler.providerRender,
                relationId,
                startTime,
            });
        }
        setLoading(true);
        handleReply(userInput, {
            aiChatService,
            aiReporter,
            chatAgentService,
            relationId,
            startTime,
            rawMessage: message,
        });
    }, [
        messageListData,
        containerRef,
        loading,
        chatFeatureRegistry,
        chatRenderRegistry,
        chatRenderRegistry.chatUserRoleRender,
    ]);
    const handleReply = React.useCallback((userInput, replayCommandProps) => {
        if (chatRenderRegistry.chatAIRoleRender) {
            replayCommandProps.renderContent = (content, status) => chatRenderRegistry.chatAIRoleRender({ content, status });
        }
        const aiMessage = (0, StreamReplyRender_1.StreamReplyRender)(userInput.message, replayCommandProps);
        if (aiMessage) {
            dispatchMessage({ type: 'add', payload: [aiMessage] });
            if (containerRef && containerRef.current) {
                containerRef.current.scrollTop = Number.MAX_SAFE_INTEGER;
            }
        }
        setLoading(false);
    }, [messageListData, chatRenderRegistry, chatRenderRegistry.chatAIRoleRender]);
    const handleClear = React.useCallback(() => {
        var _a;
        aiChatService.cancelChatViewToken();
        aiChatService.destroyStreamRequest(msgStreamManager.currentSessionId);
        aiChatService.clearSessionModel();
        (_a = containerRef === null || containerRef === void 0 ? void 0 : containerRef.current) === null || _a === void 0 ? void 0 : _a.classList.remove(SCROLL_CLASSNAME);
        dispatchMessage({ type: 'init', payload: [firstMsg] });
    }, [messageListData]);
    const handleThemeClick = (value) => {
        if (loading || loading2) {
            return;
        }
        setTheme(value);
    };
    const handleCloseChatView = React.useCallback(() => {
        layoutService.toggleSlot(common_1.AI_CHAT_VIEW_ID);
    }, [layoutService]);
    return (React.createElement("div", { id: chat_module_less_1.default.ai_chat_view },
        React.createElement("div", { className: chat_module_less_1.default.header_container },
            React.createElement("div", { className: chat_module_less_1.default.left },
                React.createElement("span", { className: chat_module_less_1.default.title }, aiAssistantName)),
            React.createElement("div", { className: chat_module_less_1.default.right },
                React.createElement(components_1.Popover, { insertClass: chat_module_less_1.default.popover_icon, id: 'ai-chat-header-clear', title: (0, ide_core_common_1.localize)('aiNative.operate.clear.title') },
                    React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: chat_module_less_1.default.action_btn, className: (0, ide_core_browser_1.getIcon)('clear'), onClick: handleClear })),
                React.createElement(components_1.Popover, { insertClass: chat_module_less_1.default.popover_icon, id: 'ai-chat-header-close', title: (0, ide_core_common_1.localize)('aiNative.operate.close.title') },
                    React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: chat_module_less_1.default.action_btn, className: (0, ide_core_browser_1.getIcon)('window-close'), onClick: handleCloseChatView })))),
        React.createElement("div", { className: chat_module_less_1.default.body_container },
            React.createElement("div", { className: chat_module_less_1.default.left_bar, id: 'ai_chat_left_container' },
                React.createElement("div", { className: chat_module_less_1.default.chat_container, ref: containerRef },
                    React.createElement(react_chat_elements_1.MessageList, { className: chat_module_less_1.default.message_list, lockable: true, toBottomHeight: '100%', 
                        // @ts-ignore
                        dataSource: messageListData }),
                    loading && (React.createElement("div", { className: chat_module_less_1.default.chat_loading_msg_box },
                        React.createElement(react_chat_elements_1.SystemMessage, { title: aiAssistantName, className: chat_module_less_1.default.smsg, 
                            // @ts-ignore
                            text: React.createElement(ChatThinking_1.ChatThinking, { status: 1 /* EMsgStreamStatus.THINKING */ }) })))),
                React.createElement("div", { className: chat_module_less_1.default.chat_input_wrap },
                    React.createElement("div", { className: chat_module_less_1.default.header_operate },
                        React.createElement("div", { className: chat_module_less_1.default.header_operate_left }, shortcutCommands.map((command) => (React.createElement(components_1.Popover, { id: `ai-chat-shortcut-${command.name}`, title: command.tooltip || command.name },
                            React.createElement("div", { className: chat_module_less_1.default.tag, onClick: () => handleThemeClick(command.nameWithSlash) }, command.name))))),
                        React.createElement("div", { className: chat_module_less_1.default.header_operate_right })),
                    React.createElement(ChatInput_1.ChatInput, { onSend: (value, agentId, command) => handleSend({ message: value, agentId, command }), disabled: loading || loading2, enableOptions: true, theme: theme, setTheme: setTheme, agentId: agentId, setAgentId: setAgentId, defaultAgentId: defaultAgentId, command: command, setCommand: setCommand, ref: chatInputRef }))))));
});
//# sourceMappingURL=chat.view.js.map