import Parser from 'web-tree-sitter';
import { Injector } from '@opensumi/di';
import * as monaco from '@opensumi/ide-monaco/lib/common';
import { SupportedLanguages, SupportedTreeSitterLanguages } from './tree-sitter/language-facts';
import { IFunctionInfo } from './tree-sitter/language-facts/base';
interface CodeBlock {
    range: monaco.IRange;
    codeBlock: string;
    type: string;
}
export declare class LanguageParser {
    private language;
    private parser;
    private parserLoaded;
    private lruCache;
    private wasmModuleManager;
    private languageFacts;
    private constructor();
    ready(): Promise<void>;
    private initializeParser;
    /**
     * 从给定的位置开始，向上遍历 AST，找到最近的代码块
     * @param node 节点
     * @param position 位置
     * @returns 代码块
     */
    private findContainingCodeBlockWithPosition;
    private findFunctionCodeBlock;
    parseAST(model: monaco.ITextModel): Promise<Parser.SyntaxNode | undefined>;
    getSyntaxNodeAsPosition(model: monaco.ITextModel, cursor: number): Promise<Parser.SyntaxNode | null>;
    findCodeBlock(model: monaco.ITextModel, position: monaco.Position): Promise<CodeBlock | null>;
    provideFunctionInfo(model: monaco.ITextModel, position: monaco.Position): Promise<IFunctionInfo | null>;
    /**
     * 从给定的位置开始，找到最近的没有语法错误的代码块
     */
    findCodeBlockWithSyntaxError(sourceCode: string, range: monaco.IRange): Promise<CodeBlock | null>;
    private static pool;
    static get(injector: Injector, language: SupportedTreeSitterLanguages): LanguageParser | undefined;
}
export declare const LanguageParserFactory: (injector: Injector) => (language: SupportedLanguages | string) => LanguageParser | undefined;
export type LanguageParserFactory = ReturnType<typeof LanguageParserFactory>;
export {};
//# sourceMappingURL=parser.d.ts.map